//
//  LACondition.h
//  LAAdditions
//
//  Created by Cyril Anger on 01/10/11.
//  Copyright (c) 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAParameter.h>
#import <LAAdditions/LATemplate.h>
#import <LAAdditions/LAOrderedMap.h>

@class LACompoundCondition, LASavedCondition, LAFiltersBarData, LARule;

@interface LACondition : LAManagedObject <LATemplateControlledObject>

@property (strong, nonatomic) LACompoundCondition *parent;
@property (getter = isLeaf, nonatomic, readonly) BOOL leaf;
@property (readonly) NSPredicate *predicate;

+ (id)insertNewConditionFromPredicate:(NSPredicate *)predicate withTemplates:(LAOrderedMap *)templates inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

@end

@interface LACompoundCondition : LACondition

@property (strong, nonatomic) LASavedCondition *savedFilter;
@property (strong, nonatomic) LAFiltersBarData *filtersBarData;
@property (strong, nonatomic) LARule *rule;
@property (readonly) LAManagedObject *unifiedParent;
@property (strong, nonatomic) NSNumber *type;
@property (nonatomic) NSCompoundPredicateType typeValue;
@property (weak) NSString *typeName;
@property (strong, nonatomic) NSOrderedSet *conditions;

+ (NSArray *)unifiedParentKeyPaths;
+ (LAOrderedMap *)conditionTemplates;

@end

@interface LACompoundCondition (CoreDataGeneratedAccessors)

- (void)insertObject:(LACondition *)value inConditionsAtIndex:(NSUInteger)index;
- (void)removeObjectFromConditionsAtIndex:(NSUInteger)index;
- (void)insertConditions:(NSArray *)values atIndexes:(NSIndexSet *)indexes;
- (void)removeConditionsAtIndexes:(NSIndexSet *)indexes;
- (void)replaceObjectInConditionsAtIndex:(NSUInteger)index withObject:(LACondition *)value;
- (void)replaceConditionsAtIndexes:(NSIndexSet *)indexes withConditions:(NSArray *)values;
- (void)addConditionsObject:(LACondition *)value;
- (void)removeConditionsObject:(LACondition *)value;
- (void)addConditions:(NSOrderedSet *)values;
- (void)removeConditions:(NSOrderedSet *)values;

@end

@interface LAComparisonCondition : LACondition

@property (strong, nonatomic) NSString *keyPath;
@property (strong, nonatomic) NSString *operator;
@property (strong, nonatomic) LAParameter *parameter;

@end
