//
//  LADocument.h
//  LAAdditions
//
//  Created by Cyril Anger on 20/06/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LADocumentSynchronizationHandler.h>
#import <LAAdditions/LAManagedObjectContextSynchronizationSessionController.h>
#import <LAAdditions/LAMainViewController.h>
#import <LAAdditions/LAPrintView.h>

@interface LADocument : NSDocument <NSWindowDelegate>

@property (strong, nonatomic, readonly) NSManagedObjectContext *managedObjectContext;
@property (strong, nonatomic, readonly) NSManagedObjectModel *managedObjectModel;
@property (getter = isProtected, nonatomic) BOOL protected;
@property (strong, nonatomic, readonly) NSArray *synchronizationHandlers;
@property (strong, nonatomic, readonly) LAManagedObjectContextSynchronizationSessionController *synchronizationSessionController;
@property (nonatomic, readonly) NSString *currentPrintInfoName;
@property (nonatomic, readonly) NSInteger numberOfMinutesInBackground;

@property (strong, nonatomic) IBOutlet LAMainViewController *mainViewController;
@property (strong, nonatomic) IBOutlet NSView *unlockView;
@property (weak, nonatomic) IBOutlet NSTextField *unlockPasswordField;
@property (strong, nonatomic) IBOutlet NSView *lockView;
@property (weak, nonatomic) IBOutlet NSTextField *lockPasswordField;
@property (weak, nonatomic) IBOutlet NSTextField *lockNewPasswordField;
@property (weak, nonatomic) IBOutlet NSTextField *lockConfirmPasswordField;
@property (weak, nonatomic) IBOutlet NSButton *lockRemoveButton;

+ (NSString *)persistentStoreName;
+ (Class)synchronizationSessionClass;

- (NSString *)persistentStoreTypeForFileType:(NSString *)fileType;
- (BOOL)configurePersistentStoreCoordinatorForURL:(NSURL *)URL ofType:(NSString *)fileType modelConfiguration:(NSString *)configuration storeOptions:(NSDictionary *)storeOptions error:(NSError * __autoreleasing *)error;
- (BOOL)migrateStoreProgressivelyAtURL:(NSURL *)URL ofType:(NSString *)fileType withVersions:(NSArray *)versions ofModel:(NSString *)modelName error:(NSError * __autoreleasing *)error;
- (BOOL)readAdditionalContentFromURL:(NSURL *)URL ofType:(NSString *)typeName error:(NSError * __autoreleasing *)error;
- (id)additionalContentForURL:(NSURL *)URL ofType:(NSString *)typeName error:(NSError * __autoreleasing *)error;
- (BOOL)writeAdditionalContent:(id)content toURL:(NSURL *)URL originalContentsURL:(NSURL *)originalContentsURL error:(NSError * __autoreleasing *)error;
- (BOOL)updateMetadata:(NSMutableDictionary *)metadata error:(NSError * __autoreleasing *)error;
- (void)restoreContentsOfURL:(NSURL *)URL completionHandler:(void (^)(BOOL success))completionHandler;
- (void)saveAndZipDocumentWithCompletionHandler:(void (^)(NSString *file, NSString *name, NSString *error))completionHandler;
- (void)unzipAndRestoreDocumentFromArchive:(NSString *)archive withCompletionHandler:(void (^)(NSString *error))completionHandler;
- (BOOL)tryToRecoverFromError:(NSError *)error;
- (void)addInitialData;
- (BOOL)checkPassword:(NSString *)password;
- (void)protectWithPassword:(NSString *)password;
- (BOOL)unlockWithPassword:(NSString *)password;
- (LADocumentSynchronizationHandler *)synchronizationHandler:(NSString *)description;
- (void)synchronizeWithDocumentAtURL:(NSURL *)URL inBackground:(BOOL)inBackground withOtherIcon:(LAImage *)otherIcon startHandler:(void (^)(LAManagedObjectContextSynchronizationSession *synchronizationSession))startHandler completionHandler:(void (^)(BOOL canceled, NSString *error))completionHandler;
- (void)configureDefaultPrintInfo:(NSPrintInfo *)printInfo forName:(NSString *)name;
- (NSPrintOperation *)printOperationWithView:(NSView *)view accessoryController:(NSViewController <NSPrintPanelAccessorizing> *)accessoryController;
- (NSPrintOperation *)PDFOperationWithView:(NSView *)view toPath:(NSString *)path;
- (void)printView:(LAPrintView *)view;
- (void)exportView:(LAPrintView *)view toPDFAtPath:(NSString *)path completionHandler:(void (^)(void))completionHandler;
- (void)printHTMLString:(NSString *)string;
- (void)exportHTMLString:(NSString *)string toPDFAtPath:(NSString *)path completionHandler:(void (^)(void))completionHandler;
- (void)documentWillBecomeActive;

- (IBAction)showUnlockView:(id)sender;
- (IBAction)hideUnlockView:(id)sender;
- (IBAction)unlock:(id)sender;
- (IBAction)showLockView:(id)sender;
- (IBAction)hideLockView:(id)sender;
- (IBAction)lock:(id)sender;
- (IBAction)removeLock:(id)sender;

@end
