//
//  LAFilter.h
//  LAAdditions
//
//  Created by Cyril Anger on 24/09/11.
//  Copyright (c) 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LACondition.h>
#import <LAAdditions/LAImageAdditions.h>

@class LAFiltersBarData;

typedef enum {
	LAFilterTypeSimple,
	LAFilterTypeList,
	LAFilterTypeString,
	LAFilterTypeDate,
	LAFilterTypeNumber
} LAFilterType;

@interface LAFilter : NSObject

@property (strong, nonatomic, readonly) NSString *name;
@property (readonly) NSString *title;
@property (nonatomic, readonly) LAFilterType type;
@property (strong, nonatomic, readonly) NSPredicate *predicate;
@property (nonatomic, readonly) NSInteger tag;
@property (getter = isEnabled, nonatomic) BOOL enabled;
@property (getter = isHidden, nonatomic) BOOL hidden;
@property (strong, nonatomic) id widget;

- (id)initWithName:(NSString *)name type:(LAFilterType)type predicate:(NSPredicate *)predicate tag:(NSInteger)tag enabled:(BOOL)enabled hidden:(BOOL)hidden;
- (BOOL)isCompatibleWithFilter:(LAFilter *)filter;

@end

@interface LASavedCondition : LAManagedObject

@property (strong, nonatomic) LAFiltersBarData *filtersBarData;
@property (strong, nonatomic) NSString *name;
@property (readonly) LAImage *icon;
@property (strong, nonatomic) LACompoundCondition *condition;

@end

@interface LAFiltersBarData : LAManagedObject

@property (strong, nonatomic) NSString *name;
@property (strong, nonatomic) NSDictionary *values;
@property (strong, nonatomic) LACompoundCondition *condition;
@property (strong, nonatomic) NSOrderedSet *savedConditions;

- (void)sortSavedConditionsAlphabetically;

@end

@interface LAFiltersBarData (CoreDataGeneratedAccessors)

- (void)insertObject:(LASavedCondition *)value inSavedConditionsAtIndex:(NSUInteger)index;
- (void)removeObjectFromSavedConditionsAtIndex:(NSUInteger)index;
- (void)insertSavedConditions:(NSArray *)values atIndexes:(NSIndexSet *)indexes;
- (void)removeSavedConditionsAtIndexes:(NSIndexSet *)indexes;
- (void)replaceObjectInSavedConditionsAtIndex:(NSUInteger)index withObject:(LASavedCondition *)value;
- (void)replaceSavedConditionsAtIndexes:(NSIndexSet *)indexes withSavedConditions:(NSArray *)values;
- (void)addSavedConditionsObject:(LASavedCondition *)value;
- (void)removeSavedConditionsObject:(LASavedCondition *)value;
- (void)addSavedConditions:(NSOrderedSet *)values;
- (void)removeSavedConditions:(NSOrderedSet *)values;

@end
