//
//  LAGraphView.h
//  LAAdditions
//
//  Created by Cyril Anger on 25/09/11.
//  Copyright (c) 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAViewAdditions.h>
#import <LAAdditions/LAScrollViewAdditions.h>
#import <LAAdditions/LAStringAdditions.h>

typedef NSString * (^LAGraphViewTitleBlock)(NSDecimalNumber *value);

@interface LAGraphView : LAView <LAScrollViewDelegate>

@property (strong, nonatomic, readonly) NSDecimalNumber *xMin;
@property (strong, nonatomic) NSDecimalNumber *xRealMin;
@property (strong, nonatomic, readonly) NSDecimalNumber *xMax;
@property (strong, nonatomic) NSDecimalNumber *xRealMax;
@property (strong, nonatomic, readonly) NSDecimalNumber *yMin;
@property (strong, nonatomic) NSDecimalNumber *yRealMin;
@property (strong, nonatomic, readonly) NSDecimalNumber *yMax;
@property (strong, nonatomic) NSDecimalNumber *yRealMax;
@property (nonatomic) CGFloat xScale;
@property (nonatomic) CGFloat yScale;
@property (nonatomic) LAEdgeInsets borderInsets;
@property (weak, nonatomic) NSString *borderInsetsComponents;
@property (nonatomic, readonly) LAEdgeInsets legendInsets;
@property (nonatomic, readonly) LARect graphBounds;
@property (nonatomic) BOOL usesCache;
@property (nonatomic, readonly) LAPoint targetPoint;

@property (weak, nonatomic) IBOutlet LAScrollView *scrollView;

- (void)reloadData;
- (LASize)computeGraphSize;
- (CGFloat)xToGraph:(NSDecimalNumber *)x;
- (CGFloat)yToGraph:(NSDecimalNumber *)y;
- (NSDecimalNumber *)graphToX:(CGFloat)x;
- (NSDecimalNumber *)graphToY:(CGFloat)y;
- (NSDecimalNumber *)yForX:(NSDecimalNumber *)x onLineFromX1:(NSDecimalNumber *)x1 y1:(NSDecimalNumber *)y1 toX2:(NSDecimalNumber *)x2 y2:(NSDecimalNumber *)y2;
- (NSDecimalNumber *)xForY:(NSDecimalNumber *)y onLineFromX1:(NSDecimalNumber *)x1 y1:(NSDecimalNumber *)y1 toX2:(NSDecimalNumber *)x2 y2:(NSDecimalNumber *)y2;
- (void)drawGraph;
- (void)drawAxes;
- (void)drawXSeparatorsAdjustingVars:(BOOL)adjustVars step:(NSDecimalNumber *)step color:(LAColor *)color dashed:(BOOL)dashed titleBlock:(LAGraphViewTitleBlock)titleBlock titleFont:(LAFont *)titleFont titleColor:(LAColor *)titleColor;
- (void)drawYSeparatorsAdjustingVars:(BOOL)adjustVars step:(NSDecimalNumber *)step color:(LAColor *)color dashed:(BOOL)dashed titleBlock:(LAGraphViewTitleBlock)titleBlock titleFont:(LAFont *)titleFont titleColor:(LAColor *)titleColor;
- (void)drawXDateSeparatorsWithColor:(LAColor *)color dashed:(BOOL)dashed dateFormat:(NSString *)dateFormat titleFont:(LAFont *)titleFont titleColor:(LAColor *)titleColor;
- (void)drawSelection;
- (void)drawComment:(NSString *)comment withFont:(LAFont *)font;
- (void)graphDoubleClickedAtPoint:(LAPoint)point;

@end

@interface NSDate (LAGraphViewAdditions)

+ (NSDate *)dateWithTimeIntervalForGraph:(NSDecimalNumber *)timeInterval;
- (NSDecimalNumber *)timeIntervalForGraph;

@end
