//
//  LASynchronizedObject.h
//  LAAdditions
//
//  Created by Cyril Anger on 08/11/12.
//  Copyright (c) 2012 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAObject.h>
#import <LAAdditions/LARequestsMap.h>
#import <LAAdditions/LAImageAdditions.h>

@interface LAObjectsRequestMap : LARequestsMap

- (void)registerObject:(id <LAObject>)object;
- (void)unregisterObject:(id <LAObject>)object;

@end

@protocol LASynchronizedObject <LAObject>

@required

@property (readonly) BOOL shouldSynchronize;
@property (readonly) NSString *synchronizationDescription;
@property (strong, nonatomic, readonly) NSDate *lastModificationDate;

+ (NSArray *)synchronizedKeyPaths;
+ (NSArray *)preparationSynchronizedKeyPaths;

- (void)updateLastModificationDate;

@end

@interface LASynchronizationObjectDescription : LAObject

@property (strong, nonatomic, readonly) NSString *description;
@property (strong, nonatomic, readonly) NSArray *changes;

- (id)initWithDescription:(NSString *)description changes:(NSArray *)changes;

@end

@interface LASynchronizationChange : LAObject

@property (strong, nonatomic, readonly) NSString *description;
@property (strong, nonatomic) NSString *value;
@property (strong, nonatomic) LAImage *image;
@property (copy, nonatomic) void (^block)(void);
@property (strong, nonatomic) NSString *otherValue;
@property (strong, nonatomic) LAImage *otherImage;
@property (copy, nonatomic) void (^otherBlock)(void);
@property (nonatomic) BOOL keepOther;

- (id)initWithDescription:(NSString *)description;
- (void)applyChoice;

@end

@interface LASynchronizationKeyPath : NSObject

@property (strong, nonatomic, readonly) NSString *keyPath;

+ (id)keyPath:(NSString *)keyPath;

- (BOOL)compareValue:(id)value withValue:(id)otherValue;
- (LASynchronizationChange *)changeBetweenObject:(id)object requestsMap:(LAObjectsRequestMap *)requestsMap andObject:(id)otherObject requestsMap:(LAObjectsRequestMap *)otherRequestsMap;
- (void)copyValueFromObject:(id)object toObject:(id)otherObject withRequestsMap:(LAObjectsRequestMap *)requestsMap;

@end

typedef NSString * (^LASynchronizationValueKeyPathDescriptionBlock)(id object, id value);
typedef LAImage * (^LASynchronizationValueKeyPathImageBlock)(id object, id value);

@interface LASynchronizationValueKeyPath : LASynchronizationKeyPath

@property (copy, nonatomic, readonly) LASynchronizationValueKeyPathDescriptionBlock descriptionBlock;
@property (copy, nonatomic, readonly) LASynchronizationValueKeyPathImageBlock imageBlock;

+ (id)keyPath:(NSString *)keyPath withDescriptionBlock:(LASynchronizationValueKeyPathDescriptionBlock)descriptionBlock;
+ (id)keyPath:(NSString *)keyPath withImageBlock:(LASynchronizationValueKeyPathImageBlock)imageBlock;
+ (id)boolKeyPath:(NSString *)keyPath;
+ (id)dateKeyPath:(NSString *)keyPath;
+ (id)amountKeyPath:(NSString *)keyPath withCurrencyKeyPath:(NSString *)currencyKeyPath;
+ (id)percentageKeyPath:(NSString *)keyPath;
+ (id)imageDataKeyPath:(NSString *)keyPath withImageKeyPath:(NSString *)imageKeyPath maxSize:(LASize)maxSize;
+ (id)colorKeyPath:(NSString *)keyPath;
+ (id)collectionKeyPath:(NSString *)keyPath;

- (id)initWithKeyPath:(NSString *)keyPath descriptionBlock:(LASynchronizationValueKeyPathDescriptionBlock)descriptionBlock;
- (id)initWithKeyPath:(NSString *)keyPath imageBlock:(LASynchronizationValueKeyPathImageBlock)imageBlock;

@end

@interface LASynchronizationReferenceKeyPath : LASynchronizationValueKeyPath

@end

@interface LASynchronizationParentKeyPath : LASynchronizationReferenceKeyPath

@property (strong, nonatomic, readonly) NSString *inverseOrderedRelationshipKeyPath;

+ (id)keyPath:(NSString *)keyPath withInverseOrderedRelationshipKeyPath:(NSString *)inverseOrderedRelationshipKeyPath;

@end

@interface LASynchronizationUnifiedParentKeyPath : LASynchronizationKeyPath

@property (strong, nonatomic, readonly) NSArray *parentKeyPaths;

+ (id)keyPath:(NSString *)keyPath withParentKeyPaths:(NSArray *)parentKeyPaths;

@end

@interface LASynchronizationRelationshipKeyPath : LASynchronizationKeyPath

@end

@interface LASynchronizationOrderedRelationshipKeyPath : LASynchronizationKeyPath

@end
